#!/usr/bin/ruby

#       GTK RC File Preprocessor
#       (c) 2008 Nathaniel Sherry
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

class Boolean

	def self.to_b string
		return true if string == true || string =~ /^true$/i
		return false if string == false || string.nil? || string =~ /^false$/i
	end

end

class ThemeTemplate

	#colour adjustment function - just for convenience
	def shade value, source

		source = "@bg_color" if source == :bg_color
		source = "@fg_color" if source == :fg_color
		source = "@selected_fg_color" if source == :sel_fg_color
		source = "@selected_bg_color" if source == :sel_bg_color
		
		return "shade(#{value}, #{source})"
		
	end
	private :shade

	def getSettingDefinitions

		settings = [
		
			Setting.new("Toolbar.Contrast",		"Toolbar background adjusted from normal background.",	0.65,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Toolbar.Shaded",		"Does the toolbar draw dividers and a background",		true,	nil,		Boolean,	lambda{|v| v}),

			Setting.new("Button.Contrast",		"Button toggle shading and hover highlights.",			1.2,	0.1..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Button.Border",		"Button/Combobox borders.",								1.0,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Button.Brightness",	"Brightness of buttons",								0.8,	0.0..2.0,	Float,		lambda{|v| v.to_f}),

			#Setting.new("Scroll.Brightness",	"Brightness of scrollbars",								0.95,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			#Setting.new("Scroll.Contrast",		"Scrollbar gradient strength.",							1.1,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			#Setting.new("Scroll.Border",		"Strength of borders on scrollbars.",					1.2,	0.0..2.0,	Float,		lambda{|v| v.to_f}),

			Setting.new("Notebook.Foreground",	"Lighten selected tab and page.",						1.1,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Notebook.Background",	"Light/Darken the unselected tabs.",					0.82,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Notebook.Padding_X",	"Horizontal padding for tabs",							4,		0..10,		Integer,	lambda{|v| v.to_i}),
			Setting.new("Notebook.Padding_Y",	"Vertical padding for tabs",							2,		0..10,		Integer,	lambda{|v| v.to_i}),
			Setting.new("Notebook.Tab_Overlap",	"Overlap of tab edges onto others",						3,		-10..10,	Integer,	lambda{|v| v.to_i}),
			
			#Setting.new("Panel.Brightness",		"Light/Darken the panel.",							1.0,	0.0..2.0,	Float,		lambda{|v| v.to_f}),

			Setting.new("Menu.Contrast",		"Menu background adjusted from normal background.",		0.65,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Menu.Selection",		"Lightens selected menu item colour.",					1.0,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Menu.Shaded",			"Does the menubar draw dividers and a background",		true,	nil,		Boolean,	lambda{|v| v}),

			Setting.new("Tree.Selection",		"Lightness of selected item in tree view.",				1.0,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Tree.Focused",			"How much brighter selection in focused treeview is.",	1.0,	0.0..2.0,	Float,		lambda{|v| v.to_f}),

			Setting.new("General.Curvature",	"How curved the widgets are.",							2.0,	0.0..20.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("General.Padding",		"How much padding certain widgets have.",				2,		0..6,		Integer,	lambda{|v| v.to_i}),

			Setting.new("Metacity.Gradient",	"Strength of the gradient the titlebar has.",			1.02,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Metacity.Highlight",	"Strength of the gradient the highlight strip has.",	1.15,	0.0..2.0,	Float,		lambda{|v| v.to_f}),
			Setting.new("Metacity.Border",		"Size of window border.",								1,		0..20,		Integer,	lambda{|v| v.to_i}),
			Setting.new("Metacity.Bottom",		"Grip at bottom of window border.",						4,		0..20,		Integer,	lambda{|v| v.to_i}),
			Setting.new("Metacity.Outline",		"Strength of outline on window border.",				0.5,	0.0..3.0,	Float,		lambda{|v| v.to_f}),


			Setting.new("Theme.Name",			"The name of this theme",								"Singularity Dark",	"",	String,		lambda{|v| v.to_s}),
			Setting.new("Theme.Author",			"The author of this theme",								"Nathaniel Sherry",	"",	String,		lambda{|v| v.to_s}),
		

		]

		set = SettingSet.new
		set.addSettings settings

		return set

	end

	def gtk_replace settings

		#Define the set of things to replace
		replace = {}


		#Toolbar
		replace["<TB>"]	= shade(settings["Toolbar.Contrast"]*0.98, :bg_color)
		replace["<TBP>"]	= shade(settings["Toolbar.Contrast"]+0.05, :bg_color)
		replace["<TBS>"]	= shade(settings["Toolbar.Contrast"]+0.05, :bg_color)
		replace["<TBA>"]	= shade(settings["Toolbar.Contrast"]+0.03, :bg_color)

		replace["<TF>"]		= "@fg_color"
		replace["<TFI>"]	= shade(0.75, replace["<TB>"])

		replace["<TB-CONTRAST>"] = "1.2"

		if settings["Toolbar.Shaded"] == true
			replace["<TOOL-SHADOW>"] = "GTK_SHADOW_OUT"
			puts "out"
		else
			replace["<TOOL-SHADOW>"] = "GTK_SHADOW_NONE"
			puts "none"
		end
		

		#Buttons
		replace["<BUTTON>"]				= shade(settings["Button.Brightness"], :bg_color)
		replace["<BUTTON-PRE>"]			= shade(settings["Button.Brightness"] * settings["Button.Contrast"], :bg_color)
		replace["<BUTTON-ACT>"]			= shade(settings["Button.Brightness"] * (1/settings["Button.Contrast"]), :bg_color)
		replace["<BUTTON-BORDER>"]		= settings["Button.Border"].to_s
		replace["<BUTTON-TEXT>"]		= settings["Button.Brightness"] > 0.5 ? "" : "

	fg[NORMAL]        = \"#FFFFFF\"
	fg[PRELIGHT]      = \"#FFFFFF\"
	fg[SELECTED]      = \"#FFFFFF\"
	fg[INSENSITIVE]   = \"#FFFFFF\"
	fg[ACTIVE]        = \"#FFFFFF\"
"
		#General
		replace["<x1>"] = settings["General.Padding"]
		replace["<y1>"] = settings["General.Padding"]
		replace["<x2>"] = settings["General.Padding"] * 2
		replace["<y2>"] = settings["General.Padding"] * 2


		#Notebook
		replace["<NOTEBOOK>"] = shade(settings["Notebook.Foreground"], :bg_color)
		replace["<NOTEBOOK-BG>"] = shade(settings["Notebook.Background"], :bg_color)
		#Since notebooks are not done with the aurora engine, we have to set curvature manually here
		replace["<CURVE-NOTEBOOK>"] = (settings["General.Curvature"] / 2).to_s

		replace["<NOTEBOOK-PAD-X>"] = settings["Notebook.Padding_X"]
		replace["<NOTEBOOK-PAD-Y>"] = settings["Notebook.Padding_Y"]
		replace["<NOTEBOOK-TAB-OVERLAP>"] = settings["Notebook.Tab_Overlap"]

		#Menu
		replace["<MENU>"] = shade(settings["Menu.Contrast"]*1.01, :bg_color)

		replace["<MENU-FG>"] = shade(1.0, :fg_color)
		replace["<MENU-SEL>"] = shade(settings["Menu.Selection"], :sel_bg_color)
		replace["<MENU-FG-INS>"] = "mix(0.8, #{replace["<MENU>"]}, #{replace["<MENU-FG>"]})"

		replace["<MENU-SHADOW>"] = "GTK_SHADOW_NONE"

		if settings["Menu.Shaded"] == true
			replace["<MENU-SHADOW>"] = "GTK_SHADOW_OUT"
		else
			replace["<MENU-SHADOW>"] = "GTK_SHADOW_NONE"
		end

		#Treeview
		replace["<TREE-BASE>"] = shade(settings["Tree.Selection"], :sel_bg_color)
		replace["<TREE-BASE-SEL>"] = shade(settings["Tree.Selection"] * settings["Tree.Focused"], :sel_bg_color)



		#Panel
		replace["<PANEL>"] = shade(settings["Panel.Brightness"], :bg_color)


		#Scrollbars
		replace["<SCROLL-BORDER>"] = settings["Scroll.Border"]
		replace["<SCROLL-GRADIENT>"] = "{#{1.0*settings["Scroll.Contrast"]},1.0, 1.0,#{1.0/settings["Scroll.Contrast"]}}" #{1.09,1.01,1.01,0.91}
		replace["<SCROLL-BRIGHT>"] = settings["Scroll.Brightness"]

		#Switch Engines

		replace["<AURORA-ENGINE>"] = "engine \"aurora\" 
			{
				menubarstyle    = 1 # 0 = flat, 1 = gradient, 2 = sunken
				curvature       = #{settings["General.Curvature"]}
				arrowsize       = 0.7 # controls combo_arrow circle size.  Diameter set by (11 + 2 * arrowsize)
				old_arrowstyle  = TRUE #set to TRUE for original circled arrows
				animation       = TRUE # FALSE = disabled, TRUE = enabled
				contrast = 1.0
			}"
			
		replace["<EQUINOX-ENGINE>"] = "engine \"equinox\" 
			{
				curvature           = #{settings["General.Curvature"]}
				menubarstyle        = 1 # 0 = flat, 1 = gradient
				toolbarstyle        = 1 # 0 = flat, 1 = gradient
				buttonstyle         = 0 # 0 = normal, 1 = glassy
				menuitemstyle       = 0 # 0 = normal, 1 = glassy
				listviewheaderstyle = 0 # 0 = normal, 1 = glassy
				scrollbarstyle      = 0 # 0 = normal, 1 = glassy, 2 = normal with handles, 3 = glassy with handles
				scalesliderstyle    = 0 # 0 = normal, 1 = glassy, 2 = normal with bullet
				checkradiostyle     = 0 # 0 = normal, 1 = glassy
				animation           = TRUE # FALSE = disabled, TRUE = enabled
				arrowsize           = 0.1 # controls combo_arrow circle size.  Diameter set by (11 + 2 * arrowsize)
			}"

		replace["<MURRINE-ENGINE>"] = "engine \"murrine\" 
			{
				roundness       = #{settings["General.Curvature"].to_i}
				animation       = TRUE # FALSE = disabled, TRUE = enabled
				contrast = 1.0
			}"

		#Name of theme
		replace["<THEME-NAME>"] = settings["Theme.Name"]
		replace["<THEME-AUTHOR>"] = settings["Theme.Author"]

		return replace

	end

	def met_replace settings


		#Define the set of things to replace
		replace = {}

		#Titlebar
		replace["[TITLEBAR-BOTTOM]"]	= "shade/gtk:bg[NORMAL]/#{settings["Menu.Contrast"]*1.015}"
		replace["[TITLEBAR-TOP]"]		= "shade/gtk:bg[NORMAL]/#{settings["Menu.Contrast"]*1.02 * settings["Metacity.Gradient"] }"
		replace["[TITLEBAR-EXTENDED]"]	= "shade/gtk:bg[NORMAL]/#{settings["Menu.Contrast"]*1.025 * settings["Metacity.Gradient"] }"


		#Highlight High/Low
		replace["[HIGHLIGHT-HIGH]"]	 = "shade/gtk:bg[SELECTED]/#{ 1.2 * settings["Metacity.Highlight"] }"
		replace["[HIGHLIGHT-LOW]"]	 = "shade/gtk:bg[SELECTED]/#{ 0.9 * (1/settings["Metacity.Highlight"]) }"


		#Window Outline
		replace["[WINDOW-OUTLINE-SELECTED]"] 		= "shade/gtk:bg[NORMAL]/#{ 0.35 * settings["Metacity.Outline"] }"
		#replace["[WINDOW-OUTLINE-SELECTED-LIGHT]"]	= "shade/gtk:bg[NORMAL]/#{ 0.35 * settings["Metacity.Outline"] }"
		replace["[WINDOW-OUTLINE-UNSELECTED]"] 		= "shade/gtk:bg[NORMAL]/#{ 0.55 * settings["Metacity.Outline"] }"


		#Titlebar Text
		replace["[TITLE-TEXT-SELECTED]"] 		= "shade/gtk:bg[NORMAL]/#{ 1.5 / settings["Menu.Contrast"] }"
		replace["[TITLE-TEXT-UNSELECTED]"] 		= "shade/gtk:bg[NORMAL]/#{ 1.2 / settings["Menu.Contrast"] }"
		replace["[TITLE-TEXT-SELECTED-ETCH]"] 	= "shade/gtk:bg[NORMAL]/#{ 0.7 * settings["Menu.Contrast"] }"


		#Border Width
		replace["[METACITY-BORDER]"] 			= "#{settings["Metacity.Border"].to_i}"


		#Bottom - Must come before Window Outline
		if settings["Metacity.Bottom"].to_i != 0
			offset = settings["Metacity.Bottom"].to_i + 2 + settings["Metacity.Border"].to_i
			replace["[METACITY-BORDER-BOTTOM]"] 	= "#{settings["Metacity.Bottom"].to_i + settings["Metacity.Border"].to_i + 2}"

			replace["[METACITY-BOTTOM-LINE-UNSELECTED]"]	= "
				<line color=\"#{replace["[WINDOW-OUTLINE-UNSELECTED]"]}\" x1=\"1\" y1=\"height-#{offset}\" x2=\"width-2\" y2=\"height-#{offset}\"/>
				<line color=\"#{replace["[TITLE-TEXT-SELECTED-ETCH]"]}\" x1=\"1\" y1=\"height-#{offset}+1\" x2=\"width-2\" y2=\"height-#{offset}+1\"/>"
				
			replace["[METACITY-BOTTOM-LINE]"]	= "
				<line color=\"#{replace["[WINDOW-OUTLINE-UNSELECTED]"]}\" x1=\"1\" y1=\"height-#{offset}\" x2=\"width-2\" y2=\"height-#{offset}\"/>
				<line color=\"#{replace["[TITLE-TEXT-SELECTED-ETCH]"]}\" x1=\"1\" y1=\"height-#{offset}+1\" x2=\"width-2\" y2=\"height-#{offset}+1\"/>"
				
		else
			replace["[METACITY-BORDER-BOTTOM]"] 	= "#{settings["Metacity.Border"].to_i}"
			replace["[METACITY-BOTTOM-LINE-UNSELECTED]"]	= ""
			replace["[METACITY-BOTTOM-LINE]"]	= ""
		end

		#Theme Name
		replace["[THEME-NAME]"] = "Duality"

		return replace

	end
	
end

