class Setting

	attr_reader :name, :shortName, :description, :type, :range, :category, :converter, :defaultValue
	attr_accessor :value

	def initialize name, description, defaultValue, range, type, converter
		@name = name
		@shortName = @name.split(".")[-1]
		@category = @name.split(".")[0..-2].join(".")
		@description = description
		@defaultValue = defaultValue
		@value = defaultValue
		@type = type
		@converter = converter
		@range = range
	end

	def getConvertedValue
		return @converter.call @value
	end

	def setConvertedValue value
		@value = @converter.call value
	end

end


class SettingSet

	def initialize
		@settings = {}
	end

	def addSettings settingList

		settingList.each{|setting|
			@settings[setting.name] = setting
		}

	end

	def each

		@settings.each{|key, value|
			yield value
		}

	end

	def toValueHash

		hash = {}
		@settings.each{|key, element|
			hash[key] = element.getConvertedValue
		}

		return hash

	end

	def size
		return @settings.size
	end

	def set settingName, value
		@settings[settingName].setConvertedValue value
	end
	
	def [] settingName
		setting = @settings[settingName]
		return 1.0 if setting == nil
		return setting.getConvertedValue
	end

	def []= settingName, value
		setting = @settings[settingName]
		return false if setting == nil
		setting.setConvertedValue value
		return true
	end

	def getCategories

		cats = []
		@settings.each{|key, value|
			cats << value.category
		}

		return cats.uniq.sort
	end

	def getSettingsForCateogry category
		settings = []
		@settings.each{|key, value|
			settings << value if value.category == category
		}

		return settings
	end

	def include? settingName
		return @settings.include? settingName
	end

end
