class Template

	attr_reader :themeName

	def initialize themeName
		@themeName = themeName
		@pathRoot = File.dirname(__FILE__) + "/Templates/"
		puts @pathRoot
		@path = @pathRoot + themeName
		@loadedTemplate = loadTemplateClass
	end

	def loadTemplateClass

		template = nil

		fileToLoad = "#{@path}/ThemeTemplate.rb"

		if File.exists? fileToLoad
			load fileToLoad
			template = ThemeTemplate.new
		end
		
		return template
		
	end

	def templateLoaded?
		return @loadedTemplate != false
	end

	def getSettings loadChanges=true

		settings = @loadedTemplate.getSettingDefinitions

		if loadChanges
			loadedSettingsHash = loadSettingsHash
			loadedSettingsHash.each{|key, value|

				settings[key] = value if settings[key] != nil
			}
		end

		return settings

	end

	def loadSettingsHash
		
		settings = {}
		
		if File.exists? "#{@path}/last-config"

			fh = File.open("#{@path}/last-config", "r")
			contents = fh.gets(nil)
			fh.close
			settings = YAML::load contents

		end
		
		return settings
		
	end

	def saveSettingsHash settings
		fh = File.open("#{@path}/last-config", "w")
		fh << YAML::dump(settings.toValueHash)
		fh.close
	end


	def doReplace sourceName, destName, replaceHash

		#open up the source file
		fh = File.open(File.expand_path(sourceName), "r")
		contents = fh.gets(nil)
		fh.close


		#Perform the replacement
		replaceHash.each{|symbol, string|
			contents.gsub! symbol, string.to_s
		}

		fh = File.open(destName, "w")
		fh << contents
		fh.close
		
	end
	private :doReplace

	def create settings
		saveSettingsHash settings

		doReplace "#{@path}/gtk-2.0/gtkrc.Duality", "#{@path}/gtk-2.0/gtkrc", @loadedTemplate.gtk_replace(settings)
		doReplace "#{@path}/metacity-1/metacity-theme-1.xml.Duality", "#{@path}/metacity-1/metacity-theme-1.xml", @loadedTemplate.met_replace(settings)

		gtk_files = `ls ./gtk-2.0/`.split("\n")

		gtk_files.each{|file|
			doReplace "#{@path}/gtk-2.0/#{file}", "#{@path}/gtk-2.0/#{file[0..-9]}", @loadedTemplate.gtk_replace(settings) if (file[-8..-1]||"").downcase == ".duality"
		}
		gtk_files.each{|file|
			`rm -rf ./gtk-2.0/#{file}` if file != "gtkrc"
		}
		gtk_files = `ls "#{@path}/gtk-2.0/"`.split("\n")
		gtk_files.each{|file|
			`cp -rf "#{@path}/gtk-2.0/#{file}" ./gtk-2.0/` if (file[-8..-1]||"").downcase != ".duality"
		}
		

		met_files = `ls ./metacity-1/`.split("\n")
		met_files.each{|file|
			`rm -rf ./metacity-1/#{file}` if file != "metacity-theme-1.xml"
		}
		met_files = `ls "#{@path}/metacity-1/"`.split("\n")
		met_files.each{|file|
			`cp -rf "#{@path}/metacity-1/#{file}" ./metacity-1/` if (file[-8..-1]||"").downcase != ".duality"
		}
		
	end

	def export settings, destPath
		
		`mkdir -p #{destPath}/gtk-2.0`
		`mkdir -p #{destPath}/metacity-1`
		
		gtk_files = `ls #{destPath}/gtk-2.0/`.split("\n")
		gtk_files.each{|file|
			`rm -rf #{destPath}/gtk-2.0/#{file}` if file != "gtkrc"
		}
		gtk_files = `ls #{@path}/gtk-2.0/`.split("\n")
		gtk_files.each{|file|
			`cp -rf "#{@path}/gtk-2.0/#{file}" #{destPath}/gtk-2.0/` if (file[-8..-1]||"").downcase != ".duality"
		}
		

		met_files = `ls #{destPath}/metacity-1/`.split("\n")
		met_files.each{|file|
			`rm -rf ./metacity-1/#{file}` if file != "metacity-theme-1.xml"
		}
		met_files = `ls #{@path}/metacity-1/`.split("\n")
		met_files.each{|file|
			`cp -rf "#{@path}/metacity-1/#{file}" #{destPath}/metacity-1/` if (file[-8..-1]||"").downcase != ".duality"
		}

	end

	def delete

		`rm -rf #{@path}`
	
	end

	def copy to

		#Copy the files
		`cp -rf #{@path} #{@pathRoot}/#{to}`

	end

end
