#!/usr/bin/ruby
#
#       Gnome Theme Preprocessor & Customizer
#       (c) 2008 Nathaniel Sherry
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#
#		0.1.0	- Initial Release
#
#		0.1.1	- Small fixes from 0.1.0
#
#		0.2.0	- Fixed appearance of Main Menu Bar on Gnome Panel
#				- Adjusted default appearance to be slightly lighter
#				- Finer control over scrollbars, added scroll and scroll_border properties
#				- Removed globalmenu property - all generated themes work with global menu now
#				- Menu contrast now matches toolbar contrast for lines in between rows of menus/toolbars
#				- Improved saving of previous setings - now default values are not saved/reloaded
#				- Restricted saving past settings to top-level gtk+metacity script to prevent confusion
#				- Cleaned up gtkrc-symbol file for consistency/readability
#				- Cleaned up code to increase readability, ease of modification/growth
#				- Added versioning information
#
#		0.2.1	- Fixed bug preventing values from being passed to metacity/gtk transform scripts
#		
#		0.3.0	- Completely rewrote gtkrc-symbol file from Clearlooks gtkrc as starting point
#				- Eliminated theme dependance on the Nodoka engine
#				- Fixed bug where symbol values were set to target default when target not yet processed.
#				- Fixed toolbar widget colour issues in applications like AbiWord and Epiphany
#				- TreeView widgets have some extra padding to prevent crowding in applications like nautilus
#				- Scale widgets now using thin handle
#				- Metacity title text beveled/etched, corners made square
#				- Metacity now takes menu as contrast value.
#				- Added curvature setting to determine curvature of widgets
#				- Added button_border parameter for button/combobox border strength
#				- Added metacity_border parameter for width of Left/Right/Bottom metacity border
#				- Added metacity_bottom parameter for extra width of etched Bottom metacity border
#				- Added metacity_outline property to adjust strength of window border outline
#				- Added tool (contrast) separate from contrast
#
#		0.4.0	- Created a single 'Setting' data structure to hold the setting names, descriptions, defaults, etc.
#				- Created a Setting data type to hold a collection of Settings, and to provide b/w compat w/ hash access
#				- Merged the transforms into single file with 2 transform functions, along with fn to get a SettingSet.
#				- Created a Template class which reads an rb file and gtk/met template files (.Duality)
#				- Made Singularity into first template
#				- Created GUI to read the SettingSet and construct a settings pane for instant feedback on changes.
#
#		0.5.0	- Bug Fixes
#				- Dark Template

require 'gtk2'
require 'libglade2'
require 'thread'

require './code/common.rb'
require './code/widgets.rb'


class Customizer
	include GetText

	attr :glade

	def initialize(path_or_data, root = nil, domain = nil, themeName="Duality")
		bindtextdomain(domain, nil, nil, "UTF-8")
		@glade = GladeXML.new(path_or_data, root, domain, nil, GladeXML::FILE) {|handler| method(handler)}
		@themeName = themeName
		@widgets = {}
		@globPath = "./code/Templates/"

		createTemplateList
		toLoad = @templateNames[0]
		if File.exists?(templateConfigFile)
			lastTemplate = `cat "#{templateConfigFile}"`.strip
			toLoad = lastTemplate if @templateNames.include?(lastTemplate)
		end
		setTemplate toLoad

		createControls

		updateTheme
		updateGTKSetting

		@glade.get_widget("checkbutton2").active = true
		@glade.get_widget("checkbutton4").active = true
		
		@glade.get_widget("toolbutton2").expand = true

		@glade.get_widget("window1").icon = Gdk::Pixbuf.new "./code/Duality-256.png"
		#@glade.get_widget("vbox6").remove @glade.get_widget("toolbar1")
		@glade.get_widget("window1").show

		@glade.get_widget("window1").signal_connect("destroy"){ Gtk.main_quit }

	end


	def createTemplateList

		list = Dir.glob "#{@globPath}*/"
		@templateNames = list.map {|item| item[@globPath.size..-2].strip}

		themeMenu = Gtk::Menu.new.show
		
		@templateNames.each{|name|

			menuItem = Gtk::MenuItem.new(name).show
			themeMenu.append menuItem
			
			menuItem.signal_connect("activate") {

				#clear the notebook
				while @glade.get_widget("notebookSettings").page != -1
					@glade.get_widget("notebookSettings").remove_page 0
				end
			
				setTemplate name
				createControls
				
				mt_update

			}
		}

		@glade.get_widget("menuItemThemes").submenu = themeMenu
		
	end


	def setTemplate template
		`echo #{template} >#{templateConfigFile}`
		@template = Template.new template
		@settings = @template.getSettings
	end
	

	def updateTheme
		
		@settings.each{|setting|
			setting.setConvertedValue @widgets[setting.name].get_value
		}
		
		@template.create @settings
		
	end


	def updateGTKSetting

		settings = Gtk::Settings.default
		settings.set_property_value "gtk-theme-name", "", "gtkrc:0"
		settings.set_property_value "gtk-theme-name", @themeName, "gtkrc:0"
		
	end


	def doInGTK &block

		Gtk.timeout_add(0){block.call; false}

	end

	def createControls

		@widgets = {}

		categories = @settings.getCategories
		categories.each{|catName|

			category = @settings.getSettingsForCateogry catName

			table = Gtk::Table.new category.size, 3, false
			table.column_spacings = 6
			table.border_width = 6


			category.each_with_index{|setting, index|

				l = Gtk::Label.new setting.shortName.gsub("_", " ").gsub(/\b\w/){$&.upcase}

				l.xalign=0.0
				l.has_tooltip=true
				l.tooltip_text = setting.description
				

				signal = ""

				if setting.type == Float
				
					s = Gtk::SpinButton.new setting.range.first, setting.range.last, 0.1
					s.digits = 2
					signal = "value-changed"

				elsif setting.type == Integer
				
					s = Gtk::SpinButton.new setting.range.first, setting.range.last, 1
					s.digits = 0
					signal = "value-changed"

				elsif setting.type == String

					s = Gtk::Entry.new
					signal = "changed"
					
				elsif setting.type == Boolean

					s = Gtk::CheckButton.new
					signal = "toggled"
					
				end

				
				
				s.set_value setting.value
				s.has_tooltip = true
				s.tooltip_text = setting.description

				s.signal_connect(signal) {

					mt_update
				}
				@widgets[setting.name] = s

				
				b = Gtk::Button.new 
				b.image = Gtk::Image.new Gtk::Stock::CLEAR, Gtk::IconSize::BUTTON
				b.signal_connect("clicked"){
					@widgets[setting.name].set_value setting.defaultValue
				}

				
							
				table.attach l, 0, 1, index, index+1, Gtk::EXPAND | Gtk::FILL, Gtk::FILL
				table.attach s, 1, 2, index, index+1, Gtk::FILL, 0
				table.attach b, 2, 3, index, index+1, Gtk::FILL, Gtk::FILL

				l.show
				s.show
				b.show
				
			}

			#insert notebook page
			@glade.get_widget("notebookSettings").append_page table, Gtk::Label.new(category[0].category)

			table.show

		}

	end


	def mt_update
		@t.kill if @t != nil
		@t = Thread.new do

			begin
				updateTheme
				doInGTK {
					updateGTKSetting
				}
			rescue => e
				puts e, e.backtrace
			end
			
		end
	end

	def about
		a = @glade.get_widget "about"
		a.logo = Gdk::Pixbuf.new("./code/Duality-256.png")
		a.show 
	end

	def hideAbout
		a = @glade.get_widget "about"
		a.visible = false 
	end

	def saveas

		@template.copy "ASDF"
		#Todo: Reload template selection
	
	end

	def export

		@template.export @settings, "~/.themes/ASDFASDF"
	
	end

	def delete
		response = fast_query "Delete #{@template.themeName}?", "Deleting this template will completely remove it from your computer.", [:"gtk-cancel", :"gtk-ok"]

		if response == 0
		#cancel

		elsif response == 1
		#ok

			@template.delete
			#Todo: Reload template selection

		end
		
	end

	def apply
		#apply theme to all apps
	end
	

	def fast_query(message, submessage, answers)
		# Create the dialog
		dialog = Gtk::MessageDialog.new(
			nil,
			Gtk::Dialog::DESTROY_WITH_PARENT,
			Gtk::MessageDialog::QUESTION,
			Gtk::MessageDialog::BUTTONS_NONE,
			message
		)



		count = 0
		answers.each{|answer|
			dialog.add_button( answer, count )
			count += 1
		}
			
		dialog.secondary_text = submessage if submessage != ""
		# Ensure that the dialog box is destroyed when the user responds.

		# Add the message in a label, and show everything we've added to the dialog.
		#dialog.vbox.add(Gtk::Label.new(message))
		response = dialog.run
		dialog.destroy
		return response
	end
	
	def templateConfigFile
	
		file = File.join( File.dirname(__FILE__), "templateSelection.conf" )
		
		return file
	
	end
	
  
end



# Main program
if __FILE__ == $0



	# Set values as your own application. 
	PROG_PATH = "./code/main.glade"
	PROG_NAME = "Duality"
	puts "*******************", `pwd`.split(File::SEPARATOR)[-1].strip
	glade = Customizer.new(PROG_PATH, nil, PROG_NAME, `pwd`.split(File::SEPARATOR)[-1].strip)

	Gtk.main

end
