#!/usr/bin/ruby

#       Gnome Theme Preprocessor & Customizer
#       (c) 2008 Nathaniel Sherry
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

require './code/common.rb'


#argument handling functions
def parseArg arg

	list = arg.split("=", 2)
	list[0] = (list[0]||"")
	return list
	
end


def processArgs file, args, settings

	
	usage file, settings if (args[0]||"").downcase == "help"

	#if this isn't a reset run, parse the args
	if (args[0]||"").downcase != "reset"


		#parse the args
		args.each{|param|

			key, value = parseArg param
			if settings.include? key
				settings.set key, (value||1)
			else
				usage file, settings, "Unknown argument: #{key}"
			end
	
		}
	
	end

end

def usage file, settings, msg = nil

	puts
	puts msg, "" if msg != nil
	puts "	Usage: #{file} <TemplateName> [option=<value> ...]"
	puts ""
	
	settings.each{|setting|

		s1 = (25 - (setting.name.length + setting.defaultValue.to_s.length+1))
		puts "#{setting.name}=#{setting.defaultValue.to_s}" + " "*s1 + setting.description

	}

	list = settings.toValueHash
	keys = list.keys
	values = list.values

	puts
	puts "Example: #{file}  #{keys[0]}=#{values[0][0]} #{keys[1]}=#{values[1][0]} #{keys[2]}=#{values[2][0]}"
	puts "Creates new theme file using values, keeps last values of missing properties"
	puts
	puts "Example: #{file} reset"
	puts "Restores the default configuration"
	puts
	exit -1

end

def usage_notheme file, message = nil
	puts
	puts message, "" if message != nil
	puts "Usage: #{file} <TemplateName> [option=<value> ...]"
	puts "Try '#{file} <TemplateName> help' for info on each template"
	puts
	exit -1
end

themeName = ARGV.shift
usage_notheme(__FILE__) if themeName == "help" || themeName == ""

template = Template.new themeName
usage_notheme __FILE__ if not template.templateLoaded?

settings = template.getSettings((ARGV[0]||"").downcase != "reset")
processArgs __FILE__, ARGV, settings

template.create settings
